/*
 * File name     : cc.h for PCI/2915
 * Version       : 2.00
 * Creation date : Aug. 1997
 * Modification  : Nov. 1997
 * Author        : M.Haseno and Y.Tanaka
 *		 : Y.Nagasaka
 *               : Nagasaki Institute of Applied Science
 * modified      : 3-Mar-1998 Carl Timmer, Jefferson Lab
 */

/* #define PBOR_SIZE               16 /* Size of the PBOR (Byte) */
/* #define IOR_SIZE                64 /* Size of the IOR (Byte) */

/* #define PBOR                0xFE80 /* Parallel Bus Operational Register */ 
/* #define IOR                 0xFF00 /* PCI Interface Operational Register */

/* #define CSR        ( PBOR + 0x00 ) /* Control/Status Register (CSR & MCR) */
/* #define CNAF       ( PBOR + 0x04 ) /* CAMAC Crate/Command Register */
/* #define TCR        ( PBOR + 0x08 ) /* Transfer Count Register */
/* #define SRR        ( PBOR + 0x0C ) /* Service Request Register (K2915-CCR)*/

/* #define FIFO       ( IOR + 0x20 ) /* Data FIFO Register */
/* #define MWA        ( IOR + 0x24 ) /* Master Write Address Register */
/* #define MWTC       ( IOR + 0x28 ) /* Master Write Transfer Count Register */
/* #define MRA        ( IOR + 0x2C ) /* Master Read Address Register */
/* #define MRTC       ( IOR + 0x30 ) /* Master Read Transfer Count Register */
/* #define ICS        ( IOR + 0x38 ) /* Interrupt Control/Status Register */ 
/* #define BMCS       ( IOR + 0x3C ) /* Bus Master Control/Status Register */

/* Macros */
#define NAFGEN(n,a,f) ( 0x3fff & (n<<9 | a <<5 | f) )

/* Camac Driver Status */
#define CC_K_READY               0
#define CC_K_BUSY                1

/* Number of Crates and so on ... */
#define CC_K_MAX_BRANCH          1
#define CC_K_MAX_CRATE           8

#define CCIOC_WAIT_LAM           1
#define CCIOC_RESET              2
#define CCIOC_CHK_BRANCH         3
#define CCIOC_SET_BRANCH         4
#define CCIOC_SET_CRATE          5
#define CCIOC_ENABLE_LAM         6
#define CCIOC_DISABLE_LAM        7
#define CCIOC_REG_DUMP           8
#define CCIOC_SINGLE             9
#define CCIOC_SETUP_DMA         10
#define CCIOC_CMD_STAT          11
#define CCIOC_PPOLL             12
#define CCIOC_ENABLE_INT        13
#define CCIOC_DISABLE_INT       14

/* Control/Status Register(CSR) field */
#define RST_INFC         0x10000000
#define CSR_MASK         0x00000540
#define RFS_IENA         0x00000100
#define DONE             0x00000080
#define CLR_PCII         0x00000020
#define PP               0x0000000A
#define GO               0x00000001
#define ERR              0x80000000  /* Error */

#define XQ_MASK          0x00030000
#define NOX              0x00020000
#define NOQ              0x00010000
#define SINGLE16         0x00002000  /* Word Size 16bit */
#define SINGLE24         0x00000000  /* Word Size 24bit */     
#define NAF_TIMEOUT      0x00080000  /* NAF transfer timeout */
#define PBUS_TIMEOUT     0x00040000  /* Parallel bus timeout */

/* 3922 is offline */
#define OFFLINE          0x2000

/* CAMAC Crate/Command Register */
#define CC_NAFRK_F08     0x00000008	/* F = 8 */
#define CC_NAFRK_F16     0x00000010	/* F = 16 */

/* Bus Master Control/Status Register */
#define FIFO_RESET       0x06000000  /* Reset Data FIFO Register */
#define BMCS_IN          0x00000010  /* Inbound FIFO */
#define BMCS_OUT         0x00000002  /* Outbound FIFO */

/* CC status Register field */
#define CC_MASK 0x000384
#define GEN_Z   0x01
#define GEN_C   0x02
#define INHIBIT 0x04
#define SRE     0x100

/* Modes */
#define CC_K_MODE_WS24    0x00
#define CC_K_MODE_WS16    0x01
#define CC_K_MODES_QSTOP  0x02
#define CC_K_MODES_QIGNR  0x04
#define CC_K_MODES_QRPT   0x06
#define CC_K_MODES_QSCAN  0x08
#define CC_K_MODE_SNGLE   0x10
#define CC_K_MODE_BLOCK   0x20

/* constant for "mode" of CAMAC block system call */
#define CC_Q_STOP     CC_K_MODES_QSTOP
#define CC_Q_IGNORE   CC_K_MODES_QIGNR
#define CC_Q_REPEAT   CC_K_MODES_QRPT
#define CC_Q_SCAN     CC_K_MODES_QSCAN

/* Defined Function names */
#define CAM_Open       COPEN
#define CAMOPN         COPEN
#define CAM_Close      CCLOSE
#define CAMCLS         CCLOSE
#define CAMACL         CAMAC
#define CAM_EnableLAM  CENLAM
#define CAM_DisableLAM CDSLAM
#define CELAM          CENLAM
#define CDLAM          CDSLAM
#define CAM_WaitLAM    CWTLAM
#define CWLAM          CWTLAM
#define CAM_EnableSIG  CESIG
#define CAM_DisableSIG CDSIG
#define CamBloWorWAIt  CDMAW
#define CBWWAI         CDMAW
#define CamBloLonWAIt  CDMAL
#define CBLWAI         CDMAL
#define CamLisExeWAIt  CLEWAI
#define CamExeWAIt     CEWAI
#define CamLisLOAD     CLLOAD
#define CamReaLIST     CRLIST
