/*            ... ... ... <<< dc.c >>> ... ... ...          */
/*               CAMAC device driver on Linux               */
/*           < Comments of installation to Linux >
     This driver can be used as a loadable module or
    as a pre-installed driver in the kernel.

    If using as a loadble module, add "-DMODULE" to the compile options.

    If using as a pre-installed driver,  do as follows.
      1  Change the directory to the <src>/drivers/char 
         and  put this source file. 
         ( <src> may be /usr/src/linux as usal.)
      2  Edit  the  mem.c.    Put   "mem_start=dc_init(mem_start);"
         before "return mem_start;" in the chr_dev_init() function.
      3  Edit the Makefile.   Add  "dc.o"  to  the  OBJS  list.
      4  On  the  <src>  directory,   recompile  the  kernel.
      5  Reboot  using  the new kernel.   Care  for the messages.

    You  may  need  following  commands to make the special file 
    and use it in user mode.
           # mknod /dev/dc c 61 0
           # chmod a+rw /dev/dc
           # mknod /dev/dc1 c 61 1
           # chmod a+rw /dev/dc1
                ...
    Enjoy.
*/

#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/signal.h>
#include <linux/errno.h>
#include <linux/fs.h>
#include <linux/ioport.h>
#include <asm/io.h>
#include <asm/segment.h>
#include <asm/system.h>
#include <asm/irq.h>

#ifdef MODULE
#include <linux/module.h>
#include <linux/version.h>
#endif

#include "camac.h"

#define HARDWARE "cc/7000"
static int    dc_int_flag = 0;
static struct wait_queue *wait_que=NULL;
static int    wakeups     = 0;

#define EV_MAX 65536
static int   dc_int_count, dc_int_lost;
static unsigned short      dc_last_data;
static int   dc_rp=0,      dc_wp=0;
static int   dc_ev_max=EV_MAX;
static unsigned short      dc_evbuff[EV_MAX];

static unsigned short      dc_outreg=0x0000;

static int  dc_open   ();
static void dc_close  ();
static int  dc_read   ();
static int  dc_write  ();
static int  dc_select ();
static void dc_int    ();
static int  dc_ioctl  ();

static struct file_operations dc_fops = {
  NULL	/* dc_lseek */,
  dc_read,
  dc_write,
  NULL	/* dc_readdir */,
  dc_select,
  dc_ioctl,
  NULL	/* dc_mmap */,
  dc_open,
  dc_close
};	/* if not defined, replace to NULL. */

static int  conv(char *tra, int val) {
  int count;
  if(val>10) {
    count=conv(tra,val/10);
    *(tra+count)=val%10+'0';
    return count+1;
  } else {
    *(tra+0)=' ';
    *(tra+1)=val%10+'0';
    return 2;
  }
}

static int cycle_wait( int stimes ) {
  register int i;
  inw(PNO+6);
  for(i=0;i<stimes;i++) if((inw(PNO+6)&0x1000)!=0) return 0;
  return -1;
}

static int  dc_open   (struct inode *inode, struct file *file) { return 0; }

static void dc_close  (struct inode *inode, struct file *file) { }

static int  dc_read   (struct inode *inode, struct file *file, 
				      char *buffer, int count) { 
  int nbytes=0; int minor_no;
  minor_no=MINOR(inode->i_rdev);
  if((dc_wp==dc_rp) && (minor_no!=0)) {
    wakeups++;
    interruptible_sleep_on(&wait_que);
    if(wakeups) wakeups--;
    if(current->signal & ~current->blocked) return -ERESTARTNOINTR;
  }
  if((minor_no==0) || (minor_no==1)) {
    nbytes=2*(dc_wp-dc_rp);
    if(nbytes>count) nbytes=((count)>>1)<<1;
    if(2*(dc_rp%EV_MAX)+nbytes>2*EV_MAX) {
      memcpy_tofs((void *)buffer,&dc_evbuff[dc_rp%EV_MAX],
				2*(EV_MAX-dc_rp%EV_MAX));
      memcpy_tofs((void *)(buffer+2*(EV_MAX-dc_rp%EV_MAX)),dc_evbuff,
				nbytes-2*(EV_MAX-dc_rp%EV_MAX));
    } else {
      memcpy_tofs((void *)buffer,&dc_evbuff[dc_rp%EV_MAX],nbytes);
    }
    dc_rp+=nbytes/2;
  }
  if(minor_no==2 || minor_no==3) {
    static int mbytes; static char tra_buff[256];
    if(mbytes==0) {
      while((dc_wp>dc_rp) && (mbytes<240)) {
        mbytes+=conv(tra_buff+mbytes,(int)(dc_evbuff[(dc_rp)%EV_MAX]));
        if(dc_evbuff[(dc_rp++)%EV_MAX]>=32768) goto moduleend;
      }
    }
  moduleend:
    if(minor_no==2) tra_buff[mbytes++]=10; else tra_buff[mbytes++]=13;
    nbytes=mbytes; if(count<nbytes) nbytes=count;
    mbytes=mbytes-nbytes;
    memcpy_tofs((void *)buffer,tra_buff,nbytes);
  }
  return nbytes;
}

static int dc_write (struct inode * inode, struct file * file, 
				    char * buffer, int count) { return 0; }

static int dc_select(struct inode * inode, struct file * file,
			      int sel_type, select_table * wait) {
  int minor_no;
  minor_no=MINOR(inode->i_rdev);
  switch(sel_type) {
  case SEL_IN:  
    if(dc_wp<=dc_rp) {
      select_wait(&wait_que,wait);
      return 0;
    }
    break;
  case SEL_OUT: break;
  case SEL_EX: {
    int lampat;
    * (unsigned short *)(&lampat)    =inw(PNO+4);
    *((unsigned short *)(&lampat)+1) =inw(PNO+6)&0x00ff;
    if(lampat==0) {
      select_wait(&wait_que,wait);
      return 0;
    }
    break;
  }
  default: break;
  }
  return 1;
}

#include "dc_int.c"

static int dc_ioctl(struct inode *inode, struct file *file,
				unsigned int cmd, unsigned long arg) {
  int ans=0;
  switch(cmd) {
  case DC_INITIAL:
    if (arg > 3) {ans=-1; break;}
    cli();
    outb(arg   ,PNO+1);
//  outw(0x0800,PNO+6);
    outw(0x0000,PNO+6); /* disable interrupt */
    if((inw(PNO+6)&0x2000)!=0) ans=0; else ans=-1;  
    sti();
    break;
  case DC_Z: 
    cli();
    outw((inw(PNO+6)&0x8c00)|0x0200,PNO+6);
    ans=cycle_wait(255);
    sti();
    break;
  case DC_C: 
    cli();
    outw((inw(PNO+6)&0x8c00)|0x0100,PNO+6);
    ans=cycle_wait(255);
    sti();
    break;
  case DC_I:
    cli();
    if(arg!=0) outw((inw(PNO+6)&0x8800)|0x0400,PNO+6);
    else       outw((inw(PNO+6)&0x8800)|0x0000,PNO+6);
    sti();
    break;
  case DC_L_P: {
    int lampat;
    *((unsigned short *)(&lampat)  ) =inw(PNO+4);
    *((unsigned short *)(&lampat)+1) =inw(PNO+6)&0x00ff;
    put_fs_long(lampat,(void *)arg);
    break;
  }
  case DC_L_E: {
    int i, lampat;
    *((unsigned short *)(&lampat)  ) =inw(PNO+4);
    *((unsigned short *)(&lampat)+1) =inw(PNO+6)&0x00ff;
    for(i=0;i<24;i++) if((lampat/(1<<i))%2) goto lamhit;
  lamhit:
    if(i<24) i++; else i=0;
    put_fs_long(i,(void*) arg);
    break;
  }
  case DC_CYCLE: {
    unsigned short na;  DCCycle dcc;
    memcpy_fromfs(&dcc,(void *)arg,sizeof(DCCycle));
    *((unsigned char *)(&na  )) = dcc.n;
    *((unsigned char *)(&na)+1) = dcc.a; 
    cli();
    outw(na,PNO+4);       
    if((dcc.f&0x18)==0x10) {
      outw(*((unsigned short *)(&dcc.data)  ),PNO+0);
      outw(*((unsigned short *)(&dcc.data)+1),PNO+2);
    }
    outw((inw(PNO+6)&0x8c00)|(unsigned short)dcc.f,PNO+6);
    outb(0,PNO+7);
    ans=cycle_wait(255);
    dcc.stat=(((~inw(PNO+6))&0x0300)>>8);
    if((dcc.f&0x18)==0x00) {
      *((unsigned short *)(&dcc.data)  )=inw(PNO+0);
      *((unsigned short *)(&dcc.data)+1)=inw(PNO+2)&0x00ff;
    }
    sti();
    memcpy_tofs((void *)arg,&dcc,sizeof(dcc));
    break;
  }
  case DC_INTTST: {
    struct pt_regs regs;
    cli();
#if LINUX_VERSION_CODE < 66374  /* 1.3.69 or earlier */
    dc_int(IRQ,&regs);
#else                           /* 1.3.70 or later   */
    dc_int(IRQ,NULL,&regs);
#endif
    sti();
    break;
  }
  case DC_INTON:
    cli();
#if defined(LAM_N) && defined(LAM_A)
    outw(LAM_A<<8|LAM_N,PNO+4);
    outw((inw(PNO+6)&0x8c00)|26,PNO+6);
    outb(0,PNO+7);
    cycle_wait(255);
#endif
    dc_int_count=0;
#if LINUX_VERSION_CODE < 66374  /* 1.3.69 or earlier */
    ans=request_irq(IRQ,dc_int,SA_INTERRUPT,HARDWARE);
#else                           /* 1.3.70 or later   */
    ans=request_irq(IRQ,dc_int,SA_INTERRUPT,HARDWARE,NULL);
#endif
    if(ans==0) dc_int_flag=1;
#if defined(OUTREG)
    dc_outreg |= 0xff00;
    outw(dc_outreg,PNO+0);
    outw(OUTREG,PNO+4);
    outw((inw(PNO+6)&0x8c00)|16,PNO+6);
    outb(0,PNO+7);
    cycle_wait(255);
#endif
    sti();
    break;
  case DC_INTOFF:
    cli();
#if defined(OUTREG)
    dc_outreg &= 0x00ff;
    outw(dc_outreg,PNO+0);
    outw(OUTREG,PNO+4);
    outw((inw(PNO+6)&0x8c00)|16,PNO+6);
    outb(0,PNO+7);
    cycle_wait(255);
#endif
#if LINUX_VERSION_CODE < 66374  /* 1.3.69 or earlier */
    if(dc_int_flag!=0)free_irq(IRQ);
#else                           /* 1.3.70 or later   */
    if(dc_int_flag!=0)free_irq(IRQ,NULL);
#endif
    dc_int_flag=0;
#if defined(LAM_N) && defined(LAM_A)
    outw(LAM_A<<8|LAM_N,PNO+4);
    outw((inw(PNO+6)&0x8c00)|24,PNO+6);
    outb(0,PNO+7);
    cycle_wait(255);
#endif
    sti();
    wake_up_interruptible(&wait_que);
    /* wakeups=0; */
    break;
  case DC_INTCLR:
    if (wakeups) wake_up(&wait_que);
    cli();
    dc_rp=0;
    dc_wp=0;
    dc_int_lost=0;
    dc_int_count=0;
    sti();
    break;
  case DC_INTCNT:
    put_fs_long(dc_int_count,(void*) arg);
    break;
  case DC_INTLOS:
    put_fs_long(dc_int_lost,(void*) arg);
    break;
  case DC_INTWRD:
    put_fs_word(dc_last_data,(void*) arg);
    break;
  case DC_INTEOF:
    dc_evbuff[(dc_wp++)%EV_MAX]=0xffff;
    if (wakeups) wake_up(&wait_que);
    break;
  default: break;
  }
  return ans;
}

#ifndef MODULE
unsigned long dc_init(unsigned long kmem_start) {
  printk("... ... ... <<< dc >>> ... ... ... installed.\n");
  printk(" dc: Buffer size is %d bytes\n", 2*dc_ev_max);
  if (register_chrdev(DC_NO,DEVICE,&dc_fops))
    printk(" dc error : Cannot register to major device %d !\n",DC_NO);
  return kmem_start;
}
#else
#if LINUX_VERSION_CODE < 66374  /* 1.3.69 or earlier */
char kernel_version[]=UTS_RELEASE;
#endif
int init_module(void) {
  printk("... ... ... <<< dc >>> ... ... ... installed.\n");
  printk(" dc: Buffer size is %d bytes\n", 2*dc_ev_max);
  if (register_chrdev(DC_NO,DEVICE,&dc_fops)) {
    printk(" dc error : Cannot register to major device %d !\n",DC_NO);
    return -ENODEV;
  }
  if (check_region(PNO,8)) {
    printk(" dc error : Cannot register IOport %x !\n",PNO);
    unregister_chrdev(DC_NO,DEVICE);
    return -ENODEV;
  }
  request_region(PNO,8,DEVICE);
#if LINUX_VERSION_CODE < 66374  /* 1.3.69 or earlier */
  if (request_irq(IRQ,dc_int,SA_INTERRUPT,HARDWARE)) {
#else                           /* 1.3.70 or later   */
  if (request_irq(IRQ,dc_int,SA_INTERRUPT,HARDWARE,NULL)) {
#endif
    printk(" dc error : Cannot register IRQ %d !\n",IRQ);
    release_region(PNO,8);
    unregister_chrdev(DC_NO,DEVICE);
    return -ENODEV;
  }
#if LINUX_VERSION_CODE < 66374  /* 1.3.69 or earlier */
  free_irq(IRQ);
#else                           /* 1.3.70 or later   */
  free_irq(IRQ,NULL);
#endif
  return 0;
}
void cleanup_module(void) {
  printk("... ... ... <<< dc >>> ... ... ... uninstalled.\n");
#if LINUX_VERSION_CODE < 66374  /* 1.3.69 or earlier */
  if(dc_int_flag!=0)free_irq(IRQ);
#else                           /* 1.3.70 or later   */
  if(dc_int_flag!=0)free_irq(IRQ,NULL);
#endif
  release_region(PNO,8);
  unregister_chrdev(DC_NO,DEVICE);
}
#endif
