#ifndef HSM_CONF_H
#define HSM_CONF_H

/*====== Edit the following values according to your system ======*/

/* VMEbus IRQ level (1..7), 
 * make sure correnpoinding T-INT jumber of 617 is set */
#define HSM_IRQ      6

/* jumper pins of address setting */
#define HSM_J05      1
#define HSM_J06      0
#define HSM_J07      0
#define HSM_J08      0
#define HSM_J09      0

/* memory size = either 1MB (0x100000) or 512kB (0x80000) */
#define HSM_MEM_SIZE 0x100000

/*
 * overflow limit: size of remaining part of block before interrupt occurs. 
 *   make sure (overflow limit) >> (scaler data size) + (max. event length)
 *   and must be n times 512 where n <= 7 (3584) .
 */
#define HSM_MOVR_LIM 1024


/*====== Followings must not be modified ? (if not power user...) ======*/

#define HSM_MEM_BASE \
(HSM_J05<<24)+(HSM_J06<<25)+(HSM_J07<<26)+(HSM_J08<<27)+(HSM_J09<<28)

/* block size in 16-bit mode */
#define HSM_BLK_SIZE (65536/sizeof(short))

/* block numbers in buffer -- last block is reserved against overrun */
#define HSM_BLK_NUM  ((HSM_MEM_SIZE/sizeof(short)\
		      /HSM_BLK_SIZE)-1)

/* header configuration */
#define HSM_HDR_LEN  4

/* block ID */
#define HSM_BLK_ID(b)  (b)[0]
/*   start comment block */
#define HSM_BID_STA  0x0001
/*   stop  comment block */
#define HSM_BID_STO  0xffff
/*   good raw data */
#define HSM_BID_RAW  0x0000
/*   bad data (serious error e.g. FERA bus overrun, DMA error, etc.) */
#define HSM_BID_BAD  0xff00
/*   ill data (inconsistent scaler with raw data, 
               due to collision of scaler PIO & raw data DMA */
#define HSM_BID_ILL  0xfe00

/* block serial number in a run */
#define HSM_BLK_SN(b)  (b)[1]

/* end of available data */
#define HSM_BLK_LEN(b) (b)[2]

/* pointer to scaler address (in 16-bit array), but redundant ? */
#define HSM_SCL_ADR(b) (b)[3]

/*
 * HSM_BLK_ID  is used in src/{recorder.c,ampler,analyser/analyzer.c}
 * HSM_BLK_LEN is used in src/analyzer/analyzer.c
 * SCL_BLK_OFS is used instead of HSM_SCL_ADR, making it redundant.  
 *
 *       Presumably an event counter in block header is more helpful, 
 *       but there is no way, in general, to obtain event counter 
 *       without using an extra scaler.  
 *       Instead, scaler at end of block will do the same job...
 */

/*
 * If header configuration is modified, 
 * source codes of "vmehb/vmehb.c" 
 *
 *   around VME_DAQ_STOP
 *          in static int vmehb_ioctl()
 * and
 *
 *   around vme_backplane_interrupt
 *          in static int vmehb_irqhandler()
 * 
 * must be also modified.
 */

#endif /* HSM_CONF_H */
