proc loadDirectory {root mask0 target0} {
    upvar $mask0 mask $target0 target
    set w $root.directories.listbox
    $w delete 0 end
    foreach i [lsort [glob -nocomplain [file dirname $mask]/{*,..}]] {
	if [file isdirectory $i] {$w insert end [file tail $i]}
    }
    set active [$w curselection]
    if {$active == ""} {set active 0}
    $w activate $active
    $w selection clear 0 end
    $w selection set active
    set w $root.files.listbox
    $w delete 0 end
    if [file isdirectory $mask] {set mask $mask/*}
    foreach i [lsort [glob -nocomplain $mask]] {
	if [file isfile      $i] {$w insert end [file tail $i]}
    }
    $w activate 0
    $w selection clear 0 end
    $w selection set active
    set target [file dirname $mask]/
    if [$root.files.listbox index end] {
	focus $root.files.listbox
    } {
	focus $root.directories.listbox
    }
}

proc changeDirectory {root mask0 target0} {
    upvar $mask0 mask $target0 target
    set w $root.directories.listbox
    focus $w
    set index [$w curselection]
    if {$index == ""} return
    set currentDirectory [pwd]
    if ![catch "cd [file dirname $mask]/[$w get $index]"]\
	    {set mask [pwd]/[file tail $mask]}
    cd $currentDirectory
    loadDirectory $root mask target
}

proc loadTarget {root mask0 target0} {
    upvar $mask0 mask $target0 target
    set w $root.files.listbox
    focus $w
    set index [$w curselection]
    if {$index == ""} return
    set target [file dirname $mask]/[$w get $index]
    focus $root.target.entry
}

proc genFileExit {root OK proc target0} {
    upvar $target0 target
    global ActiveEntry ActiveGrab
    focus $ActiveEntry
    grab release $root
    if {$ActiveGrab != ""} {grab set $ActiveGrab}
    wm withdraw $root
    if $OK {
	if [string match [set pwd [pwd]/]* $target]\
		{set target [string range $target [string length $pwd] end]}
	if {$proc != ""} {uplevel "$proc $target"}
    }
}

proc genFile {root title mask target proc write} {
    global $mask $target ActiveEntry ActiveGrab

if ![winfo exists $root] {

    toplevel $root
    wm title    $root $title
    wm iconname $root $title

    set w $root.buttons
    frame $w
    button $w.ok -text OK\
	    -command "\
	    if {\$$target == \"\" || \[file isdirectory \$$target\]}\
		{loadTarget $root $mask $target};\
	    genFileExit $root 1 \"$proc\" $target"
    button $w.cancel -text Cancel\
	    -command "genFileExit $root 0 {} {}"
    pack $w.ok $w.cancel -side left -padx 3m
    bind $root <Escape> "$w.cancel invoke"
    pack $w -side bottom -pady 3m

    set w $root.target
    frame $w
    frame $w.space -height 2
    label $w.label -text "Target File:" -underline 0
    entry $w.entry -textvariable $target
    set font [$w.entry cget -font]
    pack  $w.space -side top
    pack  $w.label -side top -anchor w
    pack  $w.entry -side bottom -fill x -expand 1
    bind  $w.entry <Return> "$root.buttons.ok invoke"
    bind  $w.entry <Up>     "focus $root.files.listbox"
    bind  $w.entry <Down>   "focus $root.files.listbox"
    bind  $root <Alt-t> "focus $w.entry"
    pack  $w -side top -fill x -padx 2

#   frame $root.space -height 3m
#   pack  $root.space -side top

    set w $root.mask
    frame $w
    label $w.label -text "Mask:" -underline 0
    entry $w.entry -textvariable $mask
    pack  $w.label -side top -anchor w
    pack  $w.entry -side bottom -fill x -expand 1
    bind  $w.entry <Return> "loadDirectory $root $mask $target"
    bind  $root <Alt-m> "focus $w.entry"
    pack  $w -side bottom -fill x -padx 2

    set w $root.directories
    frame $w
    label $w.label -text "Directories:" -underline 0
    listbox $w.listbox -yscrollcommand "$w.scrollbar set"
    scrollbar $w.scrollbar -command "$w.listbox yview"
    pack $w.label -side top -anchor w
    pack $w.scrollbar -side right -fill y
    pack $w.listbox -fill both
    bind $w.listbox <FocusIn>  {%W selection set active}
    bind $w.listbox <Double-1> "changeDirectory $root $mask $target"
    bind $w.listbox <Return>   "changeDirectory $root $mask $target"
    bind $w.listbox <space>    "changeDirectory $root $mask $target"
    bind $w.listbox <Right>    "focus $root.files.listbox"
    bind $w.listbox <Control-f> "focus $root.files.listbox"
    bind $w.listbox <KeyPress> {
	if [regexp {[a-zA-Z]} %A] {
	    for {set i 0} {$i < [%W index end]} {incr i} {
		if [string match "%A*" [%W get $i]] {
		    %W activate $i
		    %W selection clear 0 end
		    %W selection set $i
		    %W see $i
		    break
		}
	    }
	}
    }
    bind $root <Alt-d> "focus $w.listbox"
    pack $w -side left -fill y -padx 2

    set w $root.files
    frame $w
    label $w.label -text "Files:" -underline 0
    listbox $w.listbox -yscrollcommand "$w.scrollbar set"
    scrollbar $w.scrollbar -command "$w.listbox yview"
    pack $w.label -side top -anchor w
    pack $w.scrollbar -side right -fill y
    pack $w.listbox -fill both
    bind $w.listbox <FocusIn>  {%W selection set active}
    bind $w.listbox <ButtonRelease-1> "loadTarget $root $mask $target"
    bind $w.listbox <space>    "loadTarget $root $mask $target"
    bind $w.listbox <Double-1> "loadTarget $root $mask $target
				genFileExit $root 1 \"$proc\" $target"
    bind $w.listbox <Return>   "loadTarget $root $mask $target
				genFileExit $root 1 \"$proc\" $target"
    bind $w.listbox <Left>     "focus $root.directories.listbox"
    bind $w.listbox <Control-b> "focus $root.directories.listbox"
    bind $w.listbox <KeyPress> {
	if [regexp {[a-zA-Z]} %A] {
	    for {set i 0} {$i < [%W index end]} {incr i} {
		if [string match "%A*" [%W get $i]] {
		    %W activate $i
		    %W selection clear 0 end
		    %W selection set $i
		    %W see $i
		    break
		}
	    }
	}
    }
    bind $root <Alt-f> "focus $w.listbox"
    pack $w -fill both -padx 2

    wm withdraw $root
    update idletasks
    set x [expr [winfo screenwidth  $root]/2 - [winfo reqwidth  $root]/2 \
	    - [winfo vrootx [winfo parent $root]]]
    set y [expr [winfo screenheight $root]/2 - [winfo reqheight $root]/2 \
	    - [winfo vrooty [winfo parent $root]]]
    wm geom $root +$x+$y
}

set currentDirectory [pwd]
set cmd "cd \[file dirname \$$mask\]; set $mask \[pwd\]/\[file tail \$$mask\]"
catch "eval $cmd"
cd $currentDirectory
loadDirectory $root $mask $target
$root.mask.entry icursor end
$root.mask.entry xview insert
$root.target.entry icursor end
$root.target.entry xview insert
set ActiveEntry [focus]
if $write {focus $root.target.entry}
set ActiveGrab  [grab current]
grab set $root

wm deiconify $root

}

#------------------------------------------------

proc OpenFile TYPE {
    global PGMNAME
    set w .file$TYPE
    set b $w.buttons
    set f $w.file
    if ![winfo exists $w] {
	toplevel $w
	wm title $w $PGMNAME/File$TYPE
	wm iconname $w $PGMNAME/File$TYPE
	frame $b
	button $b.ok -text Ok -command "if \[file$TYPE $w\] {$b.cancel invoke}"
	button $b.cancel -text Cancel
	pack  $b.ok $b.cancel -side left -padx 2m -pady 2m
	pack  $b -side bottom
	bind $w <Escape> "$b.cancel invoke"
	if {$TYPE == "WRITE"} {
	    entry $w.comment -textvariable COMMENT -width 40
	    pack  $w.comment -side bottom -fill x -expand 1 -padx 2m
	    label $w.clabel -text Comment: -underlin 0
	    pack  $w.clabel -side bottom -anchor w
	    bind  $w <Alt-c> "focus $w.comment"
	    bind  $w.comment <Return> "$b.ok invoke"
	}
	label $w.flabel -text FileName: -underline 0
	pack  $w.flabel -side top -anchor w
	frame $f
	pack  $f -fill x -expand 1 -padx 2m
	entry $f.entry -textvariable FILENAME
	pack  $f.entry -side left -anchor w -fill x -expand 1
	bind  $w <Alt-f> "focus $f.entry"
	bind  $f.entry <Return> "$b.ok invoke"
	button $f.browse -text Browse -underline 0 -command\
		"genFile .fileBrowse $PGMNAME/Browse targetMask targetFile\
		{set FILENAME} 1"
	pack  $f.browse -side right -anchor e
	bind  $w <Alt-b> "$f.browse invoke"
	wm withdraw $w
        update idletasks
        set x [expr [winfo screenwidth  $w]/2 - [winfo reqwidth  $w]/2 \
                - [winfo vrootx [winfo parent $w]]]
        set y [expr [winfo screenheight $w]/2 - [winfo reqheight $w]/2 \
                - [winfo vrooty [winfo parent $w]]]
        wm geom $w +$x+$y
    }
    uplevel {set FILENAME "" ; set COMMENT ""}
    wm deiconify $w
    $b.cancel configure -command\
	    "grab release $w; focus [focus]; wm withdraw $w"
    grab set $w
    focus $f.entry
}
